@@ -48,7 +48,8 @@
 #' \item{\code{p.value}}{The p-value(s) of the unit root test(s);}
 #' \item{\code{rejections}}{For \code{"mult_htest"} only. A vector with logical indicators for each time series whether the null hypothesis of a unit root is rejected (\code{TRUE}) or not (\code{FALSE}). This is only supplied when an optional significance level is given, otherwise \code{NULL} is returned;}
 #' \item{\code{details}}{For \code{"mult_htest"} only. The details of the performed tests in a matrix containing parameter estimate. test statistic and p-value for each time series.}
-#' \item{\code{series.names}}{For \code{"mult_htest"} only. The names of the series that the tests are performed on.}
+#' \item{\code{series.names}}{For \code{"mult_htest"} only. The names of the series that the tests are performed on;}
+#' \item{\code{specifications}}{The specifications used in the test(s).}
 #' @section Warnings:
 #' The function may give the following warnings.
 #' \describe{
@@ -111,22 +112,24 @@ boot_ur <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999, block_l
     }
   }
 
+  spec <- list("bootstrap" = bootstrap, "B" = B, "block_length" = inputs$inputs$l,
+               "ar_AWB" = inputs$inputs$ar_AWB, "level" = level,
+               "union" = union, "union_quantile" = inputs$inputs$union_quantile,
+               "deterministics" = inputs$inputs$deterministics,
+               "detrend" = inputs$inputs$detrend, "min_lag" = min_lag,
+               "max_lag" = inputs$inputs$p_max, "criterion" = inputs$inputs$criterion,
+               "criterion_scale" = inputs$inputs$criterion_scale)
+
   # Results
   if (union) { # Union test
-    iADFout <- iADF_cpp(test_i = inputs$test_stats, t_star = inputs$test_stats_star, level = inputs$level)
+    iADFout <- iADF_cpp(test_i = inputs$test_stats, t_star = inputs$test_stats_star,
+                        level = inputs$level)
     iADFout <- cbind(rep(NA, nrow(iADFout)), iADFout)
     if (NCOL(data) > 1) {
       rownames(iADFout) <- var_names
       colnames(iADFout) <- c("gamma", "tstat", "p-value")
     }
     # Parameter estimates, tstats and p-values. Note: Parameter Estimates not defined for union test
-
-    if (NCOL(data) == 1){ # boot_union
-      method_name <- "Bootstrap Union test on a single time series"
-    } else {
-      method_name <- "Bootstrap Union tests on each individual series (no multiple testing correction)"
-    }
-
   } else { # No union test
     iADFout <- iADF_cpp(test_i = matrix(inputs$tests_i[1, ], nrow = 1),
                         t_star = matrix(inputs$t_star[ , 1, ], nrow = B), level = inputs$level)
@@ -135,12 +138,6 @@ boot_ur <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999, block_l
       rownames(iADFout) <- var_names
       colnames(iADFout) <- c("gamma", "statistic", "p.value")
     }
-
-    if (NCOL(data) == 1){ # boot_adf
-      method_name <- "Bootstrap ADF test on a single time series"
-    } else {
-      method_name <- "Bootstrap ADF tests on each individual series (no multiple testing correction)"
-    }
   }
 
   if (!is.null(level)) {
@@ -150,12 +147,21 @@ boot_ur <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999, block_l
   }
 
   if (NCOL(data) > 1) {
+    if (union) {
+      method_name <- paste(bootstrap, "Bootstrap Union test on each individual series (no multiple testing correction)")
+    } else {
+      method_name <- paste(bootstrap, "Bootstrap", inputs$inputs$name,
+                           " test ( with" , inputs$inputs$deterministics,
+                           ") on each individual series (no multiple testing correction)")
+    }
     boot_ur_output <- list(method = method_name, data.name = data_name,
                            null.value =  c("gamma" = 0), alternative = "less",
                            estimate = iADFout[, 1], statistic = iADFout[, 2],
                            p.value = iADFout[, 3], rejections = rej_H0,
-                           details = NULL, series.names = var_names)
+                           details = NULL, series.names = var_names , specifications = spec)
     class(boot_ur_output) <- c("bootUR", "mult_htest")
+
+
   } else {
     param <- drop(iADFout[1, 1])
     attr(param, "names") <- "gamma"
@@ -163,9 +169,18 @@ boot_ur <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999, block_l
     attr(iADFtstat, "names") <- "tstat"
     p_val <- drop(iADFout[1, 3])
     attr(p_val, "names") <- "p-value"
+
+    if (union) {
+      method_name <- paste(bootstrap, "Bootstrap Union test on a single time series")
+    } else {
+      method_name <- paste(bootstrap, "Bootstrap", inputs$inputs$detrend,
+                           " test ( with" , inputs$inputs$deterministics,
+                           ") on a single time series")
+    }
     boot_ur_output <- list(method = method_name, data.name = var_names,
                            null.value = c("gamma" = 0), alternative = "less",
-                           estimate = param, statistic = iADFtstat, p.value = p_val)
+                           estimate = param, statistic = iADFtstat, p.value = p_val,
+                           specifications = spec)
     class(boot_ur_output) <- c("bootUR", "htest")
   }
 
@@ -197,7 +212,8 @@ boot_ur <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999, block_l
 #' \item{\code{alternative}}{A character string specifying the direction of the alternative hypothesis relative to the null value. The alternative postulates that the series is stationary;}
 #' \item{\code{estimate}}{The estimated value of the (gamma) parameter of the lagged dependent variable in the ADF regression.;}
 #' \item{\code{statistic}}{The value of the test statistic of the unit root test;}
-#' \item{\code{p.value}}{The p-value of the unit root test.}
+#' \item{\code{p.value}}{The p-value of the unit root test;}
+#' \item{\code{specifications}}{The specifications used in the test.}
 #' @section Errors and warnings:
 #' \describe{
 #' \item{\code{Error: Multiple time series not allowed. Switch to a multivariate method such as boot_ur, or change argument data to a univariate time series.}}{The function is a simple wrapper around \code{\link{boot_ur}} to facilitate use for single time series. It does not support multiple time series, as \code{\link{boot_ur}} is specifically suited for that.}
@@ -275,7 +291,8 @@ boot_adf <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999,
 #' \item{\code{alternative}}{A character string specifying the direction of the alternative hypothesis relative to the null value. The alternative postulates that the series is stationary;}
 #' \item{\code{estimate}}{For the union test, the estimated value of the (gamma) parameter of the lagged dependent variable in the ADF regression is not defined, hence NA is given;}
 #' \item{\code{statistic}}{The value of the test statistic of the unit root test;}
-#' \item{\code{p.value}}{The p-value of the unit root test.}
+#' \item{\code{p.value}}{The p-value of the unit root test;}
+#' \item{\code{specifications}}{The specifications used in the test.}
 #' @section Errors and warnings:
 #' \describe{
 #' \item{\code{Error: Multiple time series not allowed. Switch to a multivariate method such as boot_ur, or change argument data to a univariate time series.}}{The function is a simple wrapper around \code{\link{boot_ur}} to facilitate use for single time series. It does not support multiple time series, as \code{\link{boot_ur}} is specifically suited for that.}
@@ -344,7 +361,8 @@ boot_union <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999, bloc
 #' \item{\code{p.value}}{A vector with \code{NA} values, as p-values are not available for the FDR method;}
 #' \item{\code{rejections}}{A vector with logical indicators for each time series whether the null hypothesis of a unit root is rejected (\code{TRUE}) or not (\code{FALSE});}
 #' \item{\code{details}}{The details of the performed tests in a matrix containing for each step the test statistics and critical value, up to non-rejection.}
-#' \item{\code{series.names}}{The names of the series that the tests are performed on.}
+#' \item{\code{series.names}}{The names of the series that the tests are performed on;}
+#' \item{\code{specifications}}{The specifications used in the test(s).}
 #' @section Errors and warnings:
 #' \describe{
 #' \item{\code{Error: Resampling-based bootstraps MBB and SB cannot handle missing values.}}{If the time series in \code{data} have different starting and end points (and thus some series contain \code{NA} values at the beginning and/or end of the sample, the resampling-based moving block bootstrap (MBB) and sieve bootstrap (SB) cannot be used, as they create holes (internal missings) in the bootstrap samples. Switch to another bootstrap method or truncate your sample to eliminate \code{NA} values.}
@@ -407,18 +425,26 @@ boot_fdr <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999, block_
     }
   }
 
+  spec <- list("bootstrap" = bootstrap, "B" = B, "block_length" = inputs$inputs$l,
+               "ar_AWB" = inputs$inputs$ar_AWB, "FDR_level" = FDR_level, "union" = union,
+               "deterministics" = inputs$inputs$deterministics,
+               "detrend" = inputs$inputs$detrend, "min_lag" = min_lag,
+               "max_lag" = inputs$inputs$p_max, "criterion" = inputs$inputs$criterion,
+               "criterion_scale" = inputs$inputs$criterion_scale)
+
   if (union) { # Union Tests
     bFDRout <- FDR_cpp(test_i = inputs$test_stats, t_star = inputs$test_stats_star,
                        level = inputs$level)
     estimates <- rep(NA, NCOL(data))
     tstats <- drop(inputs$test_stats)
-    method_name <- "Bootstrap Union Tests with False Discovery Rate control"
+    method_name <- paste(bootstrap, "Bootstrap Union test with False Discovery Rate control")
   } else { # No Union Tests
       bFDRout <- FDR_cpp(test_i = matrix(inputs$tests_i[1, ], nrow = 1),
                          t_star = inputs$t_star[ , 1,], level = inputs$level)
       estimates <- t(inputs$param_i)
       tstats <- drop(inputs$tests_i[1, ])
-      method_name <- "Bootstrap ADF Tests with False Discovery Rate control"
+      method_name <- paste(bootstrap, "Bootstrap", inputs$inputs$name, " tests ( with" ,
+                           inputs$inputs$deterministics, ") with False Discovery Rate control")
   }
   rej_H0 <- matrix(bFDRout$rej_H0 == 1, nrow = NCOL(data))
   rownames(rej_H0) <- var_names
@@ -432,7 +458,8 @@ boot_fdr <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999, block_
   fdr_output <- list(method = method_name, data.name = data_name,
                      null.value =  c("gamma" = 0), alternative = "less",
                      estimate = estimates, statistic = tstats, p.value = p_vals,
-                     rejections = rej_H0, details = FDR_seq, series.names = var_names)
+                     rejections = rej_H0, details = FDR_seq, series.names = var_names,
+                     specifications = spec)
   class(fdr_output) <- c("bootUR", "mult_htest")
 
   return(fdr_output)
@@ -458,8 +485,9 @@ boot_fdr <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999, block_
 #' \item{\code{statistic}}{The value of the test statistic of the unit root tests;}
 #' \item{\code{p.value}}{A vector with \code{NA} values, as p-values per inidividual series are not available.The p-value for each test in the sequence can be found in \code{details};}
 #' \item{\code{rejections}}{A vector with logical indicators for each time series whether the null hypothesis of a unit root is rejected (\code{TRUE}) or not (\code{FALSE});}
-#' \item{\code{details}}{The details of the performed tests in a matrix containing for each step the stationary units undr the null and alternative hypothesis, the test statistic and the p-value.}
-#' \item{\code{series.names}}{The names of the series that the tests are performed on.}
+#' \item{\code{details}}{The details of the performed tests in a matrix containing for each step the stationary units undr the null and alternative hypothesis, the test statistic and the p-value;}
+#' \item{\code{series.names}}{The names of the series that the tests are performed on;}
+#' \item{\code{specifications}}{The specifications used in the tests.}
 #' @section Errors and warnings:
 #' \describe{
 #' \item{\code{Error: Resampling-based bootstraps MBB and SB cannot handle missing values.}}{If the time series in \code{data} have different starting and end points (and thus some series contain \code{NA} values at the beginning and/or end of the sample, the resampling-based moving block bootstrap (MBB) and sieve bootstrap (SB) cannot be used, as they create holes (internal missings) in the bootstrap samples. Switch to another bootstrap method or truncate your sample to eliminate \code{NA} values.}
@@ -523,18 +551,28 @@ boot_sqt <- function(data, data_name = NULL, steps = 0:NCOL(data), bootstrap = "
     }
   }
 
+  spec <- list("steps" = steps, "bootstrap" = bootstrap, "B" = B, "block_length" = inputs$inputs$l,
+               "ar_AWB" = inputs$inputs$ar_AWB, "SQT_level" = SQT_level, "union" = union,
+               "deterministics" = inputs$inputs$deterministics,
+               "detrend" = inputs$inputs$detrend, "min_lag" = min_lag,
+               "max_lag" = inputs$inputs$p_max, "criterion" = inputs$inputs$criterion,
+               "criterion_scale" = inputs$inputs$criterion_scale)
+
   if (union) { # Union Tests
     BSQTout <- BSQT_cpp(pvec = inputs$p_vec, test_i = inputs$test_stats,
                         t_star = inputs$test_stats_star, level = inputs$level)
     estimates <- rep(NA, NCOL(data))
     tstats <- drop(inputs$test_stats)
-    method_name <- "Bootstrap Sequential Quantile Union Test"
+    method_name <- paste(bootstrap, "Bootstrap Sequential Quantile Union test")
   } else { # No Union Tests
     BSQTout <- BSQT_cpp(pvec = inputs$p_vec, test_i = matrix(inputs$tests_i[1, ], nrow = 1),
                         t_star = inputs$t_star[ , 1,], level = inputs$level)
     estimates <- t(inputs$param_i)
     tstats <- drop(inputs$tests_i[1, ])
-    method_name <- "Bootstrap Sequential Quantile ADF Test"
+    method_name <- paste(bootstrap, "Bootstrap Sequential Quantile",
+                         inputs$inputs$name, " test ( with" ,
+                         inputs$inputs$deterministics,")")
+
   }
   rej_H0 <- matrix(BSQTout$rej_H0 == 1, nrow = NCOL(data))
   rownames(rej_H0) <- var_names
@@ -548,7 +586,8 @@ boot_sqt <- function(data, data_name = NULL, steps = 0:NCOL(data), bootstrap = "
   sqt_output <- list(method = method_name, data.name = data_name,
                      null.value =  c("gamma" = 0), alternative = "less",
                      estimate = estimates, statistic = tstats, p.value = p_vals,
-                     rejections = rej_H0, details = BSQT_seq, series.names = var_names)
+                     rejections = rej_H0, details = BSQT_seq, series.names = var_names,
+                     specifications = spec)
   class(sqt_output) <- c("bootUR", "mult_htest")
   return(sqt_output)
 }
@@ -565,7 +604,8 @@ boot_sqt <- function(data, data_name = NULL, steps = 0:NCOL(data), bootstrap = "
 #' \item{\code{alternative}}{A character string specifying the direction of the alternative hypothesis relative to the null value. The alternative postulates that the series is stationary;}
 #' \item{\code{estimate}}{For the union test, the estimated value of the (gamma) parameter of the lagged dependent variable in the ADF regression is not defined, hence NA is given;}
 #' \item{\code{statistic}}{The value of the test statistic of the unit root test;}
-#' \item{\code{p.value}}{The p-value of the unit root test.}
+#' \item{\code{p.value}}{The p-value of the unit root test;}
+#' \item{\code{specifications}}{The specifications used in the test.}
 #' @section Errors and warnings:
 #' \describe{
 #' \item{\code{Error: Resampling-based bootstraps MBB and SB cannot handle missing values.}}{If the time series in \code{data} have different starting and end points (and thus some series contain \code{NA} values at the beginning and/or end of the sample, the resampling-based moving block bootstrap (MBB) and sieve bootstrap (SB) cannot be used, as they create holes (internal missings) in the bootstrap samples. Switch to another bootstrap method or truncate your sample to eliminate \code{NA} values.}
@@ -616,16 +656,26 @@ boot_panel <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999,
   if (is.null(data_name)) {
     data_name <- deparse(substitute(data))
   }
+
+  spec <- list("bootstrap" = bootstrap, "B" = B, "block_length" = inputs$inputs$l,
+               "ar_AWB" = inputs$inputs$ar_AWB,"union" = union,
+               "union_quantile" = inputs$inputs$union_quantile,
+               "deterministics" = inputs$inputs$deterministics,
+               "detrend" = inputs$inputs$detrend, "min_lag" = min_lag,
+               "max_lag" = inputs$inputs$p_max, "criterion" = inputs$inputs$criterion,
+               "criterion_scale" = inputs$inputs$criterion_scale)
+
   if (union) { # Union Test
     GM_test <- mean(inputs$test_stats)
     t_star <- rowMeans(inputs$test_stats_star)
     p_val <- mean(t_star < GM_test)
-    method_name <- "Panel Bootstrap Group-Mean Union Test"
+    method_name <- paste("Panel", bootstrap, "Bootstrap Group-Mean Union test")
   } else { # No Union Test
     GM_test <- rowMeans(inputs$tests_i)
     t_star <- apply(inputs$t_star, 1:2, mean)
     p_val <- sapply(1, function(i){mean(t_star[, i] < GM_test[i])})
-    method_name <- "Panel Bootstrap Group-Mean Test"
+    method_name <- paste("Panel", bootstrap, "Bootstrap Group-Mean", inputs$inputs$name,
+                         " test ( with" , inputs$inputs$deterministics,")")
   }
 
   attr(GM_test, "names") <- "tstat"
@@ -634,7 +684,8 @@ boot_panel <- function(data, data_name = NULL, bootstrap = "AWB", B = 1999,
   attr(p_val, "names") <- "p-value"
   panel_output <- list(method = method_name, data.name = data_name,
                        null.value = c("gamma" = 0), alternative = "less",
-                       estimate = gamma_hat, statistic = GM_test, p.value = p_val)
+                       estimate = gamma_hat, statistic = GM_test, p.value = p_val,
+                       specifications = spec)
   class(panel_output) <- c("bootUR", "htest")
   return(panel_output)
 }
